@@ -11,11 +11,11 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <linux/atm.h>
-#include <linux/atmsap.h>
 #include <assert.h>
 
 /* Local includes */
 #include "atm.h"
+#include "atmsap.h"
 #include "load.h"
 #include "dump.h"
 #include "connect.h"
