@@ -15,9 +15,9 @@
 #include <netinet/in.h>
 
 #include <linux/atm.h>
-#include <linux/atmsap.h>
 
 /* Local includes */
+#include "atmsap.h"
 #include "dump.h"
 #include "lane.h"
 #include "load.h"
