@@ -65,6 +65,7 @@
 #include "kernel_itf.h"
 #include "lec_ctrl.h"
 #include "addr_reg.h"
+#include "lec_arp.h"
 
 #define EMOD MOD_CM
 #define EINST "conn.c"
@@ -106,6 +107,9 @@ typedef struct _conn_t_ {
   int type;                /* We made, they made, listen socket */
   void *p_packet;          /* Save packets address, needed in callback */
   CONN_INFO *conn_info;    /* Listen socket conn_info */
+  unsigned char atm_address[ATM_ESA_LEN]; 
+                           /* Destination address, not in listen or kernel
+                             sockets */
   struct _conn_t_ *next;
   struct _conn_t_ *previous;
 } Conn_t;
@@ -247,7 +251,6 @@ static Conn_t*
 conn_already_exists(unsigned char *atm_addr, Conn_t *current)
 {
   Conn_t *conn;
-  struct sockaddr_atmsvc them;
   int len;
 
   conn = connlist;
@@ -256,12 +259,8 @@ conn_already_exists(unsigned char *atm_addr, Conn_t *current)
   while (conn) {
     if (conn != current && conn->type != LISTENING && 
 	conn->type != KERNEL_SOCK && conn->status != RELEASED) {
-      if (getpeername(conn->fd, (struct sockaddr*)&them, &len) < 0) {
-	EVENT(EM_NERR,("Can't getpeername:%s\n",strerror(errno)));
-      } else {
-	if (memcmp(them.sas_addr.prv, atm_addr, ATM_ESA_LEN) == 0)
-	  return conn;
-      }
+      if (memcmp(conn->atm_address, atm_addr, ATM_ESA_LEN) == 0)
+       return conn;
     }
     conn= conn->next;
   }
@@ -286,7 +285,6 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   int ret;
   struct sockaddr_atmsvc us;
   struct atm_blli blli;
-  struct atmlec_ioc ioc_data;
 
   EVENT(EM_EVENT,("Outgoing call setup\n"));
   
@@ -434,22 +432,6 @@ cm_sap_svc_setup (HANDLE sap_handle, HANDLE conn_context,
   switch (p_conn_info->blli.l3.tr9577.snap[4]) { /* AAAAAARRRGGHHH! */
   case 0x02:
   case 0x03:
-    memcpy(ioc_data.atm_addr, p_conn_info->addr.sas_addr.prv, ATM_ESA_LEN);
-    ioc_data.dev_num = sap->itf_num;
-    ioc_data.receive = 0;
-    EVENT(EM_DEBUG,("About to ioctl(s, ATMLEC_DATA, &ioc_data)\n"));
-    if (ioctl(s, ATMLEC_DATA, &ioc_data)<0) {
-      EVENT(EM_SERR,("Can't change socket into LE data socket:%s\n",
-		     strerror(errno)));      
-      if (is_data_direct(p_conn_info)) {
-	/* Try to remove possible entry in kernel */
-	lc_addr_delete(conn_context, p_conn_info->addr.sas_addr.prv);
-      }
-      close(s);
-      *p_conn_handle = NULL;
-      return STATUS_K_ATM_RESOURCES;
-    }
-    EVENT(EM_DEBUG,("ioctl done\n"));
     break;
   case 0x04:
   case 0x05:
@@ -536,6 +518,7 @@ cm_sap_xmt_vc (HANDLE conn_handle, void *p_data,
   /* Failure */
   if (ret<0) {
     EVENT(EM_NERR,("Write failed: %s\n",strerror(errno)));
+    conn->status = RELEASED;
     return STATUS_K_CONGESTED; /* Asdf */
   } 
   if (EMASK & EM_XCTRL)
@@ -830,10 +813,32 @@ conn_check_incoming(fd_set *fds)
       } else if (conn->type == LISTENING) { /* Incoming call */
       } else if (conn->status == CONNECTING) { /* Call is ready */
 	memset(&addr,0,sizeof(addr));
-	if (!connect(conn->fd, (struct sockaddr*)&addr,sizeof(addr))) {
-	  EVENT(EM_EVENT,("Nonblocking connect() :%s\n",strerror(errno)));
+	/*
+	if (!connect(conn->fd, (struct sockaddr*)&addr,sizeof(addr)) &&
+	  errno != EINPROGRESS) {
+	  */
+	if (connect(conn->fd, (struct sockaddr*)&addr, sizeof(addr))<0) {
+	  EVENT(EM_SERR,("Nonblocking connect() :%s\n",strerror(errno)));
+	  lc_addr_delete(conn->conn_context, conn->atm_address);
+	  conn->status = RELEASED;
+	} else {
+	  EVENT(EM_DEBUG,("Call connected\n"));
+	  conn->status = CONNECTED;
+	  memcpy(ioc_data.atm_addr, conn->atm_address, ATM_ESA_LEN);
+	  sap = conn->sap_handle;
+	  ioc_data.dev_num = sap->itf_num;
+	  ioc_data.receive = 0;
+	  EVENT(EM_DEBUG,("About to ioctl(s, ATMLEC_DATA, &ioc_data)\n"));
+	  if (ioctl(conn->fd, ATMLEC_DATA, &ioc_data)<0) {
+	    EVENT(EM_SERR,("Can't change socket into LE data socket:%s\n",
+			   strerror(errno)));
+	    conn->status = RELEASED;
+	    lc_addr_delete(conn->conn_context, conn->atm_address);
+	  } else {
+	    EVENT(EM_DEBUG,("ioctl done\n"));
+	    kernel_flush_xmt(conn->atm_address);
+	  }
 	}
-	conn->status = CONNECTED;
       } else { /* Data coming in */
 	nbytes = read(conn->fd, buffer, BUFSIZE);
 	if (nbytes <0) {
@@ -846,8 +851,7 @@ conn_check_incoming(fd_set *fds)
 	    EVENT(EM_EVENT, ("Closing connection\n"));
 	  }
 	} else if (nbytes==0) {
-	  /* EOF, connection closed */
-	  
+         /* EOF, connection closed */    
 	  conn->status = RELEASED;
 	  EVENT(EM_EVENT,("Connection closed\n"));
 	} else {
@@ -987,6 +991,10 @@ list_add_conn(HANDLE sap_handle, HANDLE conn_context, long age_limit,
   conn->conn_context = conn_context;
   conn->info = info;
   conn->age_limit = age_limit;
+  if (info)
+    memcpy(conn->atm_address, info->addr.sas_addr.prv, ATM_ESA_LEN);
+  else
+    memset(conn->atm_address, 0, ATM_ESA_LEN);
 
   conn->next = connlist;
   conn->previous = NULL;
