@@ -26,6 +26,7 @@
 /* Message types */
 
 #define QMSG_SETUP		0x05	/* SETUP */
+#define QMSG_ALERTING		0x01	/* ALERTING */
 #define QMSG_CALL_PROC		0x02	/* CALL PROCEEDING */
 #define QMSG_CONNECT		0x07	/* CONNECT */
 #define QMSG_CONN_ACK		0x0f	/* CONNECT ACKNOWLEDGE */
@@ -33,11 +34,13 @@
 #define QMSG_REL_COMP		0x5a	/* RELEASE COMPLETE */
 #define QMSG_RESTART		0x46	/* RESTART */
 #define QMSG_REST_ACK		0x4e	/* RESTART ACKNOWLEDGE */
+#define QMSG_NOTIFY		0x6e	/* NOTIFY */
 #define QMSG_STATUS		0x7d	/* STATUS */
 #define QMSG_STATUS_ENQ		0x75	/* STATUS ENQUIRY */
 #define QMSG_ADD_PARTY		0x80	/* ADD PARTY */
 #define QMSG_ADD_PARTY_ACK	0x81	/* ADD PARTY ACKNOWLEDGE */
 #define QMSG_ADD_PARTY_REJ	0x82	/* ADD PART REJECT */
+#define QMSG_PARTY_ALERT	0x85	/* PARTY ALERTING */
 #define QMSG_DROP_PARTY		0x83	/* DROP PARTY */
 #define QMSG_DROP_PARTY_ACK	0x84	/* DROP PARTY ACKNOWLEDGE */
 
