@@ -977,7 +977,7 @@ static void data_sd(SSCOP_DSC *dsc,int s,void *msg,int length) /* 20.38-39 */
 	dsc->vr_h = MOD24(s+1);
 	return;
     }
-    if (queue_lookup(&dsc->rx_buf,s)) queue_put(&dsc->rx_buf,buf);
+    if (!queue_lookup(&dsc->rx_buf,s)) queue_put(&dsc->rx_buf,buf);
     else {
 	buffer_discard(buf);
 	start_error_recov(dsc,'Q');
