@@ -131,7 +131,6 @@ static int bind_check(SOCKET *sock,struct sockaddr_atmsvc *local)
 
 static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 {
-    SOCKET *next;
     int error;
 
     switch (msg->type) {
@@ -184,37 +183,33 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 	    START_TIMER(sock,T303);
 	    new_state(sock,ss_connecting);
 	    return;
-	case as_accept: /* LISTENING only */
-	    if (sock->state != ss_listening) break;
-	    /* becomes INDICATED or ZOMBIE */
-	    if (!sock->listen) {
-		diag(COMPONENT,DIAG_WARN,
-		  "socket 0x%lx got accept with empty listen queue",msg->vcc);
-		return;
-	    }
-	    next = sock->listen;
-	    sock->listen = next->listen;
-	    next->listen = NULL;
-	    if (next->state == ss_zombie) {
+	case as_accept:
+	    if (sock->state == ss_zombie) {
 		SEND_ERROR(msg->vcc,-ECONNRESET); /* -ERESTARTSYS ? */
-		free_sock(next);
+		free_sock(sock);
 		return;
 	    }
-	    if (next->state != ss_indicated) {
-		sock = next; /* for error reporting */
-		break;
-	    }
-	    next->id = msg->vcc;
-	    error = send_connect(next);
+	    if (sock->state != ss_indicated) break;
+	    error = send_connect(sock);
 	    if (!error) {
-		START_TIMER(next,T313);
-		new_state(next,ss_accepting);
+		START_TIMER(sock,T313);
+		new_state(sock,ss_accepting);
 		return;
 	    }
-	    SEND_ERROR(next->id,error);
-	    send_release(next,0); /* @@@ */
-	    START_TIMER(next,T308_1);
-	    new_state(next,ss_wait_rel);
+	    SEND_ERROR(sock->id,error);
+	    send_release(sock,0); /* @@@ */
+	    START_TIMER(sock,T308_1);
+	    new_state(sock,ss_wait_rel);
+	    return;
+	case as_reject: /* ZOMBIE or INDICATED */
+	    if (sock->state == ss_zombie) {
+		free_sock(sock);
+		return;
+	    }
+	    if (sock->state != ss_indicated) break;
+	    send_release(sock,ATM_CV_CALL_REJ); /* @@@ should use msg->reply */
+	    START_TIMER(sock,T308_1);
+	    new_state(sock,ss_wait_rel);
 	    return;
 	case as_listen: /* NULL */
 	    {
@@ -234,9 +229,9 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		sock->state = ss_listening;
 		return;
 	    }
-	case as_close: /* all but ACCEPTING and WAIT_REL */
-	    if (sock && (sock->state == ss_accepting || sock->state ==
-	      ss_wait_rel)) break;
+	case as_close: /* all but INDICATED, ZOMBIE, and WAIT_REL */
+	    if (sock && (sock->state == ss_indicated || sock->state ==
+	      ss_zombie || sock->state == ss_wait_rel)) break;
 	    switch (sock ? sock->state : ss_null) {
 		case ss_listening:
 		case ss_zombie:
@@ -249,6 +244,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		case ss_rel_req:
 		    return;
 		case ss_connecting:
+		case ss_accepting:
 		    STOP_TIMER(sock);
 		    /* fall through */
 		case ss_connected:
@@ -256,7 +252,6 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 			diag(COMPONENT,DIAG_INFO,"Active close (CR 0x%06X)",
 			  sock->call_ref);
 		    /* fall through */
-		case ss_indicated:
 		case ss_hold:
 		    send_release(sock,
 #ifdef UNI31
@@ -266,8 +261,8 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 #endif
 		      );
 		    START_TIMER(sock,T308_1);
-		    new_state(sock,sock->state == ss_connecting ||
-		      sock->state == ss_connected ? ss_rel_req : ss_wait_rel);
+		    new_state(sock,sock->state != ss_hold ? ss_rel_req :
+		      ss_wait_rel);
 		    return;
 		case ss_rel_ind:
 		    send_release_complete(sock->call_ref,0); /* @@@ */
@@ -287,6 +282,27 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 }
 
 
+static void dispatch_listen(SOCKET *sock,struct atmsvc_msg *msg)
+{
+    SOCKET *next;
+
+    if (!sock) {
+	diag(COMPONENT,DIAG_WARN,"message %s is incompatible with state %s "
+	  "(%d)",as_name[msg->type],state_name[ss_null],ss_null);
+	return;
+    }
+    if (!(next = sock->listen)) {
+	diag(COMPONENT,DIAG_WARN,
+	  "socket 0x%lx got accept with empty listen queue",msg->vcc);
+	return;
+    }
+    sock->listen = next->listen;
+    next->listen = NULL;
+    next->id = msg->vcc;
+    dispatch(next,msg);
+}
+
+
 void itf_load(int itf)
 {
     char buf[MAX_ATM_ADDR_LEN+1];
@@ -318,6 +334,7 @@ void itf_load(int itf)
 
 void from_kernel(struct atmsvc_msg *msg,int size)
 {
+    void (*dispatcher)(SOCKET *,struct atmsvc_msg *);
     SOCKET *curr;
     struct atm_blli **this;
     int bllis,i;
@@ -326,12 +343,17 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	itf_load(msg->pvc.sap_addr.itf);
 	return;
     }
-    for (curr = sockets; curr; curr = curr->next)
-	if (msg->vcc == curr->id) break;
-    if (!curr)
+    if (msg->type != as_accept && msg->type != as_reject) {
+	dispatcher = dispatch;
+	for (curr = sockets; curr; curr = curr->next)
+	    if (msg->vcc == curr->id) break;
+    }
+    else {
+	dispatcher = dispatch_listen;
 	for (curr = sockets; curr; curr = curr->next)
 	    if (msg->listen_vcc == curr->id && curr->state == ss_listening)
 		break;
+    }
     diag(COMPONENT,DIAG_DEBUG,"FROM KERNEL: %s for socket 0x%lx (0x%lx/0x%lx) "
       "in state %s",as_name[msg->type],(unsigned long) curr,msg->vcc,
       msg->listen_vcc,state_name[curr ? curr->state : ss_null]);
@@ -343,5 +365,5 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	this = &msg->blli[i].next;
     }
     *this = NULL;
-    dispatch(curr,msg);
+    dispatcher(curr,msg);
 }
