@@ -6,9 +6,9 @@
 #ifndef PROTO_H
 #define PROTO_H
 
-#include <linux/atmsap.h>
 #include <linux/atmsvc.h>
 
+#include "atmsap.h"
 #include "atmd.h"
 
 
@@ -64,6 +64,7 @@ extern unsigned char q_buffer[];
 extern int net;
 extern int pretty;
 extern int sig_pcr;
+extern const char *sig_qos;
 extern const char *dump_dir;
 
 
