@@ -78,6 +78,9 @@ static void setup_call(unsigned long call_ref)
     this->call_ref = call_ref;
     if (q_present(&in_dsc,QF_ep_ref))
 	this->ep_ref = q_fetch(&in_dsc,QF_ep_ref);
+#ifdef CISCO
+    else
+#endif
     send_call_proceeding(this);
     this->local = new_sap;
     this->qos = new_qos;
