@@ -210,22 +210,20 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap,
     if (!*sap->sas_addr.prv) return -EDESTADDRREQ;
     else q_write(dsc,QF_cdpn_esa,(void *) sap->sas_addr.prv,ATM_ESA_LEN);
 	/* improve @@@ */
+    if (qos->txtp.traffic_class == ATM_UBR || qos->rxtp.traffic_class ==
+      ATM_UBR) q_assign(dsc,QF_best_effort,0);
     switch (qos->txtp.traffic_class) {
 	case ATM_NONE:
-	    if (qos->rxtp.traffic_class == ATM_CBR)
-		q_assign(dsc,QF_fw_pcr_01,0);
+	    q_assign(dsc,QF_fw_pcr_01,0);
 	    break;
 	case ATM_UBR:
-	    diag(COMPONENT,DIAG_DEBUG,"UBR");
-	    q_assign(dsc,QF_fw_pcr_01,ATM_OC3_PCR);
-	    q_assign(dsc,QF_bw_pcr_01,ATM_OC3_PCR);
-	    q_assign(dsc,QF_best_effort,0);
-	    break;
+	    /* fall through */
 	case ATM_CBR:
 	    /* here's a bit of policy: send the highest value we have */
 	    pcr = SELECT_TOP_PCR(qos->txtp);
-	    diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d (%d..%d)",pcr,
+	    diag(COMPONENT,DIAG_DEBUG,"fwd %d (%d..%d)",pcr,
 	      qos->txtp.min_pcr,qos->txtp.max_pcr);
+	    if (pcr == ATM_MAX_PCR) pcr = ATM_OC3_PCR;
 	    q_assign(dsc,QF_fw_pcr_01,pcr);
 	    break;
         default:
@@ -233,16 +231,17 @@ int sap_encode(Q_DSC *dsc,const struct sockaddr_atmsvc *sap,
 	      qos->txtp.traffic_class);
 	    return -EINVAL;
     }
-    switch (qos->txtp.traffic_class == ATM_UBR ? ATM_NONE :
-      qos->rxtp.traffic_class) {
+    switch (qos->rxtp.traffic_class) {
 	case ATM_NONE:
-	    if (qos->txtp.traffic_class == ATM_CBR)
-		q_assign(dsc,QF_bw_pcr_01,0);
+	    q_assign(dsc,QF_bw_pcr_01,0);
 	    break;
+	case ATM_UBR:
+	    /* fall through */
 	case ATM_CBR:
 	    pcr = SELECT_TOP_PCR(qos->rxtp);
-	    diag(COMPONENT,DIAG_DEBUG,"CBR bwd %d (%d..%d)",pcr,
+	    diag(COMPONENT,DIAG_DEBUG,"bwd %d (%d..%d)",pcr,
 	      qos->rxtp.min_pcr,qos->rxtp.max_pcr);
+	    if (pcr == ATM_MAX_PCR) pcr = ATM_OC3_PCR;
 	    q_assign(dsc,QF_bw_pcr_01,pcr);
 	    break;
         default:
@@ -361,23 +360,24 @@ struct sockaddr_atmsvc *sap_decode(Q_DSC *dsc,struct atm_qos *qos)
     }
     else {
 	qos->txtp.traffic_class = qos->rxtp.traffic_class = ATM_CBR;
-	qos->txtp.max_pcr = qos->rxtp.max_pcr = 0;
-	/* unbalanced decoding - always sets upper bound */
-	if (q_present(dsc,QF_fw_pcr_01)) {
-	    qos->rxtp.min_pcr = 0;
-	    qos->rxtp.max_pcr = q_fetch(dsc,QF_fw_pcr_01);
-	}
-	if (q_present(dsc,QF_bw_pcr_01)) {
-	    qos->txtp.min_pcr = 0;
-	    qos->txtp.max_pcr = q_fetch(dsc,QF_bw_pcr_01);
-	}
-	if (!qos->txtp.max_pcr) qos->txtp.traffic_class = ATM_NONE;
-	if (!qos->rxtp.max_pcr) qos->rxtp.traffic_class = ATM_NONE;
-	diag(COMPONENT,DIAG_DEBUG,"CBR fwd %d..%d bwd %d..%d",
-	  qos->rxtp.min_pcr,qos->rxtp.max_pcr,qos->txtp.min_pcr,
-	  qos->txtp.max_pcr);
-	/* SHOULD ... fail call if anything is missing ... @@@ */
+	diag(COMPONENT,DIAG_DEBUG,"CBR");
     }
+    qos->txtp.max_pcr = qos->rxtp.max_pcr = 0;
+    /* unbalanced decoding - always sets upper bound */
+    if (q_present(dsc,QF_fw_pcr_01)) {
+	qos->rxtp.min_pcr = 0;
+	qos->rxtp.max_pcr = q_fetch(dsc,QF_fw_pcr_01);
+    }
+    if (q_present(dsc,QF_bw_pcr_01)) {
+	qos->txtp.min_pcr = 0;
+	qos->txtp.max_pcr = q_fetch(dsc,QF_bw_pcr_01);
+    }
+    if (!qos->txtp.max_pcr) qos->txtp.traffic_class = ATM_NONE;
+    if (!qos->rxtp.max_pcr) qos->rxtp.traffic_class = ATM_NONE;
+    diag(COMPONENT,DIAG_DEBUG,"fwd %d..%d bwd %d..%d",
+      qos->rxtp.min_pcr,qos->rxtp.max_pcr,qos->txtp.min_pcr,
+      qos->txtp.max_pcr);
+    /* SHOULD ... fail call if anything is missing ... @@@ */
     if (q_present(dsc,QF_bw_max_sdu))
 	qos->txtp.max_sdu = q_fetch(dsc,QF_bw_max_sdu);
     if (q_present(dsc,QF_fw_max_sdu))
