@@ -126,16 +126,13 @@ static void append_tp(const struct atm_trafprm *tp)
 }
 
 
-void qd_dump(const char *msg,...)
+static void qd_vdump(const char *msg,va_list ap)
 {
     const char *prev,*end;
-    va_list ap;
 
     if (new_line) append("  ");
     prev = string;
-    va_start(ap,msg);
     vappend(msg,ap);
-    va_end(ap);
     if (!prev) prev = string;
     if (!prev) new_line = 1;
     else {
@@ -146,6 +143,28 @@ void qd_dump(const char *msg,...)
 }
 
 
+void qd_dump(const char *msg,...)
+{
+    va_list ap;
+
+    va_start(ap,msg);
+    qd_vdump(msg,ap);
+    va_end(ap);
+}
+
+
+void qd_report(int severity,const char *msg,...)
+{
+    va_list ap;
+
+    if (severity > Q_ERROR) return;
+    va_start(ap,msg);
+    qd_vdump(msg,ap);
+    va_end(ap);
+    qd_dump("\n");
+}
+
+
 static void print_q2931(void *msg,int size)
 {
     Q_DSC dsc;
