@@ -39,7 +39,7 @@ int main(int argc,char **argv)
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),T2A_PVC) < 0)
 	usage(argv[0]);
     memset(&qos,0,sizeof(qos));
-    qos.rxtp.class = ATM_UBR;
+    qos.rxtp.traffic_class = ATM_UBR;
     qos.rxtp.max_sdu = BSIZE;
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
 	perror("setsockopt SO_ATMQOS");
