@@ -483,15 +483,23 @@ printf("sbs=%d\n",sockbufsize);
 		err("accept");
 	    
 	    {
-		struct sockaddr_in peer;
+		struct sockaddr_atmsvc peer;
 		int peerlen = sizeof(peer);
 		if (getpeername(fd, (struct sockaddr_in *) &peer, 
 				&peerlen) < 0) {
 		    err("getpeername");
 		}
-		if (atm) fprintf(stderr,"ttcp-r: accept\n");
+		if (atm) {
+		    char name[MAX_ATM_ADDR_LEN+1];
+
+		    if (atm2text(name,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
+		      &peer,A2T_NAME | A2T_PRETTY) < 0)
+			strcpy(name,"<invalid>");
+		    fprintf(stderr,"ttcp-r: accept from %s\n",name);
+		}
 		else fprintf(stderr,"ttcp-r: accept from %s\n",
-			    inet_ntoa(peer.sin_addr));
+			    inet_ntoa(((struct sockaddr_in *) &peer)->
+			      sin_addr));
 	    }
 	    
 	    if (options)  {
